@@ -188,21 +188,6 @@ the cache file. Your OS will want to write those dirty pages back to
 the file on the actual physical disk, but the rate it does that at is
 very OS dependent.
 
-In Linux, you have some control over how the OS writes those pages
-back using a number of parameters in /proc/sys/vm
-
-  dirty_background_ratio
-  dirty_expire_centisecs
-  dirty_ratio
-  dirty_writeback_centisecs
-
-How you tune these depends heavily on your setup.
-
-As an interesting point, if you use a highmem linux kernel, a change
-between 2.6.16 and 2.6.20 made the kernel flush memory a LOT more.
-There's details in this kernel mailing list thread:
-L<http://www.uwsg.iu.edu/hypermail/linux/kernel/0711.3/0804.html>
-
 In most cases, people are not actually concerned about the persistence
 of data in the cache, and so are happy to disable writing of any cache
 data back to disk at all. Baically what they want is an in memory only
@@ -218,9 +203,20 @@ And we put all our cache files on there. The tmpfs filesystem is smart
 enough to only use memory as required by files actually on the tmpfs,
 so making it 1G in size doesn't actually use 1G of memory, it only uses
 as much as the cache files we put on it. In all cases, we ensure that
-we never run out of real memory, so the cache files effectively act 
+we never run out of real memory, so the cache files effectively act
 just as named access points to shared memory.
 
+If you do put the cache file on a filesystem backed by a real disk,
+in Linux you have some control over how the OS writes those pages
+back using a number of parameters in /proc/sys/vm
+
+  dirty_background_ratio
+  dirty_expire_centisecs
+  dirty_ratio
+  dirty_writeback_centisecs
+
+How you tune these depends heavily on your setup.
+
 Some people have suggested using anonymous mmaped memory. Unfortunately
 we need a file descriptor to do the fcntl locking on, so we'd have
 to create a separate file on a filesystem somewhere anyway. It seems
@@ -300,7 +296,7 @@ use strict;
 use warnings;
 use bytes;
 
-our $VERSION = '1.55';
+our $VERSION = '1.59';
 
 require XSLoader;
 XSLoader::load('Cache::FastMmap', $VERSION);
@@ -1002,6 +998,26 @@ sub get_and_set {
   return wantarray ? ($Value, $DidStore) : $Value;
 }
 
+=item I<exists($Key)>
+
+Search cache for given Key. Returns false if not found or true
+if found. This will *not* call the I<read_cb> if not found.
+
+This is also an optimisation over C<get()> as it will not uncompress or
+deserialize the value if found in the cache.
+
+=cut
+sub exists {
+  my ($Self, $Cache) = ($_[0], $_[0]->{Cache});
+
+  # Hash value, lock page, read result
+  my ($HashPage, $HashSlot) = fc_hash($Cache, $_[1]);
+  my $Unlock = $Self->_lock_page($HashPage);
+  my (undef, $Flags, $Found, $ExpireOn) = fc_read($Cache, $HashSlot, $_[1]);
+
+  return $Found;
+}
+
 =item I<remove($Key, [ \%Options ])>
 
 Delete the given key from the cache
@@ -1057,6 +1073,35 @@ sub get_and_remove {
   return wantarray ? ($Value, $DidDel) : $Value;
 }
 
+=item I<expire($Key)>
+
+Explicitly expire the given $Key. For a cache in write-back mode, this
+will cause the item to be written back to the underlying store if dirty,
+otherwise it's the same as removing the item. 
+
+=cut
+sub expire {
+  my ($Self, $Cache) = ($_[0], $_[0]->{Cache});
+
+  # Hash value, lock page, read result
+  my ($HashPage, $HashSlot) = fc_hash($Cache, $_[1]);
+  my $Unlock = $Self->_lock_page($HashPage);
+  my ($Val, $Flags, $Found) = fc_read($Cache, $HashSlot, $_[1]);
+
+  # If we found it, remove it
+  if ($Found) {
+    (undef, $Flags) = fc_delete($Cache, $HashSlot, $_[1]);
+  }
+  $Unlock = undef;
+
+  # If it's dirty, write it back
+  if (($Flags & FC_ISDIRTY) && (my $write_cb = $Self->{write_cb})) {
+    eval { $write_cb->($Self->{context}, $_[1], $Val); };
+  }
+
+  return $Found;
+}
+
 =item I<clear()>
 
 Clear all items from the cache
